package edu.ub.tfg.josepperez.classificadorapp;


public class FourierManager {
    private int blockSize;

    public FourierManager(int blockSize){
        this.blockSize = blockSize;
    }

    public double[] calculaEspectre(short[] buffer, int bufferReadResult){
        Complex[] arrayComplex = new Complex[blockSize];
        double[] arrayTemp = new double[blockSize];
        double[] espectre = new double[blockSize];
        for (int i = 0; i < blockSize; i++) {

            arrayTemp[i] = (double) buffer[i] / 32768.0; // signed 16 bit
            arrayComplex[i] = new Complex(arrayTemp[i],0);
        }

        Complex[] espectreComplex = this.fft(arrayComplex);
        for(int i=0;i<blockSize/2;i++){
            espectre[i]=espectreComplex[i].re();
        }
        return espectre;
    }

    private Complex[] fft(Complex[] x) {
        int N = x.length;

        // base case
        if (N == 1) return new Complex[] { x[0] };

        // radix 2 Cooley-Tukey FFT
        if (N % 2 != 0) { throw new RuntimeException("N is not a power of 2"); }

        // fft of even terms
        Complex[] even = new Complex[N/2];
        for (int k = 0; k < N/2; k++) {
            even[k] = x[2*k];
        }
        Complex[] q = fft(even);

        // fft of odd terms
        Complex[] odd  = even;  // reuse the array
        for (int k = 0; k < N/2; k++) {
            odd[k] = x[2*k + 1];
        }
        Complex[] r = fft(odd);

        // combine
        Complex[] y = new Complex[N];
        for (int k = 0; k < N/2; k++) {
            double kth = -2 * k * Math.PI / N;
            Complex wk = new Complex(Math.cos(kth), Math.sin(kth));
            y[k]       = q[k].plus(wk.times(r[k]));
            y[k + N/2] = q[k].minus(wk.times(r[k]));
        }
        return y;
    }
}
