package edu.ub.tfg.josepperez.classificadorapp;

import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaRecorder;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

public class MainActivity extends AppCompatActivity implements View.OnClickListener {

    private double[][] sv;
    private double[] coefs;
    private Classificador classificador= new Classificador();
    ImageView recordButton;
    ImageView stopButton;
    Gravacio gravacio;
    private ImageView imageView;
    Bitmap bitmap;
    private Canvas canvas;
    Paint paint;
    private int tipusSo;
    ArrayList<double[]> espectres;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        sv = llegirVectors();
        coefs = llegirCoeficients();
        classificador = new Classificador(sv, coefs);
        gravacio = new Gravacio(true,this);
        recordButton = (ImageView) this.findViewById(R.id.record_button);
        recordButton.setOnClickListener(this);
        espectres = new ArrayList<double[]>();
    }

    public void onClick(View v) {

        stopButton = (ImageView) this.findViewById(R.id.stop_button);

        gravacio.execute();

        imageView = (ImageView) this.findViewById(R.id.imageView1);
        bitmap = Bitmap.createBitmap((int) 512, (int) 100,Bitmap.Config.ARGB_8888);
        canvas = new Canvas(bitmap);
        paint = new Paint();
        paint.setColor(ContextCompat.getColor(this.getApplicationContext(), R.color.blue_green));
        imageView.setImageBitmap(bitmap);
        stopButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                gravacio.cancel(true);
                Gravacio.setStarted(false);
                for(int i=0;i<espectres.size();i++){
                    double maxim = maximAbsolut(espectres.get(i));
                    for (int j = 0; j < Gravacio.BLOCKSIZE / 2; j++) {
                        espectres.get(i)[j] = Math.abs(espectres.get(i)[j])/maxim;
                    }
                }
                tipusSo = classificador.classifica(espectres);
                while(!gravacio.getClassificat()){}
                startActivityResultat(tipusSo);
                gravacio.setClassificat(false);
            }
        });

    }

    public void refresh(double[]... toTransform){
        canvas.drawColor(Color.BLACK);

        for (int i = 0; i < toTransform[0].length; i++) {
            int x = i;
            int downy = (int) (100 - (toTransform[0][i] * 10));
            int upy = 100;

            canvas.drawRect(x * 5, downy, x * 5 + 4, upy, paint);
        }

        espectres.add(gravacio.getEspectre());

        imageView.invalidate();
    }

    public void startActivityResultat(int resultat){
        Intent i = new Intent(MainActivity.this,Resultat.class);
        i.putExtra("RESULTAT", resultat);
        startActivity(i);
        this.finish();

    }

    public void onBackPressed(){
        gravacio.cancel(true);
        Gravacio.setStarted(false);
        this.finish();
    }

    private double maximAbsolut(double[] espectre) {
        double maxim = Math.abs(espectre[0]);
        for(int i=0; i<espectre.length; i++){
            if(Math.abs(espectre[i]) > maxim){
                maxim = Math.abs(espectre[i]);
            }
        }
        return maxim;
    }

    private double[][] llegirVectors(){

        double vectors[][] = new double[Classificador.NV][Classificador.MIDA_VECTOR];

        InputStream is = getResources().openRawResource(getResources().getIdentifier("raw/vectors", "raw", getPackageName()));
        BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(is));


        for (int i = 0; i < Classificador.NV; i++) {
            for (int j = 0; j < Classificador.MIDA_VECTOR; j++) {
                String line = null;
                try {
                    line = bufferedReader.readLine();
                } catch (IOException e) {
                    e.printStackTrace();
                }
                double parseDouble = Double.parseDouble(line);
                vectors[i][j] = parseDouble;
            }
        }



        return vectors;
    }

    private double[] llegirCoeficients(){

        double coeficients[] = new double[Classificador.NV];

        InputStream is = getResources().openRawResource(getResources().getIdentifier("raw/coeficients", "raw", getPackageName()));
        BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(is));

        for(int i=0;i<Classificador.NV;i++) {
            String line = null;
            try {
                line = bufferedReader.readLine();
            } catch (IOException e) {
                e.printStackTrace();
            }
            double parseDouble = Double.parseDouble(line);
            coeficients[i] = parseDouble;
        }

        return coeficients;
    }

    protected void onResume(){
        super.onResume();
        espectres = new ArrayList<double[]>();
    }


}